/*
	Demonstration of the CL_Mouse polling interface
*/

#include <ClanLib/core.h>
#include <ClanLib/application.h>
#include <ClanLib/display.h>
#include <ClanLib/gl.h>

class MouseApp : public CL_ClanApplication
{
public:
	virtual int main(int, char **)
	{
		// Create a console window for text-output if not available
		CL_ConsoleWindow console("Console");
		console.redirect_stdio();

		try
		{
			CL_SetupCore::init();
			CL_SetupDisplay::init();
			CL_SetupGL::init();

			CL_DisplayWindow window("Mouse Example", 640, 480);
			
			// Loop until the user presses Escape
			while (CL_Keyboard::get_keycode(CL_KEY_ESCAPE) == false)
			{
				// Clear display
				CL_Display::clear(CL_Color(0, 0, 128));
			
				// Flip front and backbuffer - this makes the changes visible
				CL_Display::flip();
			
				CL_System::sleep(10);

				// Update keyboard input and handle system events
				CL_System::keep_alive();

				if(CL_Mouse::get_keycode(CL_MOUSE_LEFT))
					std::cout << "Left pressed at " << CL_Mouse::get_x() << "," << CL_Mouse::get_y() << std::endl;
				if(CL_Mouse::get_keycode(CL_MOUSE_MIDDLE))
					std::cout << "Middle pressed" << std::endl;
				if(CL_Mouse::get_keycode(CL_MOUSE_RIGHT))
					std::cout << "Right pressed" << std::endl;
			}
		}
		catch (CL_Error err)
		{
			std::cout << "Exception caught: " << err.message.c_str() << std::endl;
		}

		// Display console close message and wait for a key
		console.display_close_message();

		CL_SetupGL::deinit();
		CL_SetupDisplay::deinit();
		CL_SetupCore::deinit();

		return 0;
	}
} app;
