/*  $Id: xml_token.cpp,v 1.3 2003/08/15 12:42:46 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Core/precomp.h"
#include "API/Core/XML/xml_token.h"
#include "xml_token_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_XMLToken construction:

CL_XMLToken::CL_XMLToken() : impl(new CL_XMLToken_Generic)
{
	impl->ref_count++;
}
	
CL_XMLToken::CL_XMLToken(const CL_XMLToken &copy) : impl(copy.impl)
{
	if (impl) impl->ref_count++;
}
	
CL_XMLToken::~CL_XMLToken()
{
	if (impl && --impl->ref_count == 0) delete impl;
}
	
/////////////////////////////////////////////////////////////////////////////
// CL_XMLToken attributes:

CL_XMLToken::TokenType CL_XMLToken::get_type() const
{
	return impl->type;
}

CL_XMLToken::TokenVariant CL_XMLToken::get_variant() const
{
	return impl->variant;
}

const std::string &CL_XMLToken::get_name() const
{
	return impl->name;
}

const std::string &CL_XMLToken::get_value() const
{
	return impl->value;
}

const std::vector< std::pair<std::string, std::string> > &CL_XMLToken::get_attributes() const
{
	return impl->attributes;
}

const std::string &CL_XMLToken::get_attribute(const std::string &name) const
{
	int size = impl->attributes.size();
	for (int i=0; i<size; i++)
	{
		if (impl->attributes[i].first == name)
		{
			return impl->attributes[i].second;
		}
	}

	static std::string null_attribute;
	return null_attribute;
}

/////////////////////////////////////////////////////////////////////////////
// CL_XMLToken operations:

CL_XMLToken &CL_XMLToken::operator =(const CL_XMLToken &copy)
{
	if (impl == copy.impl) return *this;

	if (impl && --impl->ref_count == 0) delete impl;
	impl = copy.impl;
	if (impl) impl->ref_count++;
	return *this;
}

void CL_XMLToken::set_type(TokenType type)
{
	impl->type = type;
}

void CL_XMLToken::set_variant(TokenVariant variant)
{
	impl->variant = variant;
}

void CL_XMLToken::set_name(const std::string &name)
{
	impl->name = name;
}

void CL_XMLToken::set_value(const std::string &value)
{
	impl->value = value;
}

void CL_XMLToken::set_attribute(const std::string &name, const std::string &value)
{
	int size = impl->attributes.size();
	for (int i=0; i<size; i++)
	{
		if (impl->attributes[i].first == name)
		{
			impl->attributes[i].second = value;
			return;
		}
	}
	impl->attributes.push_back(std::pair<std::string, std::string>(name, value));
}

/////////////////////////////////////////////////////////////////////////////
// CL_XMLToken implementation:
