/*  $Id: display_window_description.cpp,v 1.5 2003/07/20 22:31:49 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/
	
#include "Display/display_precomp.h"
#include "API/Display/display_window_description.h"
#include "display_window_description_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_DisplayWindowDescription construction:

CL_DisplayWindowDescription::CL_DisplayWindowDescription() : impl(0)
{
	impl = new CL_DisplayWindowDescription_Generic;
	impl->title = "Untitled clanlib window";
	impl->position = CL_Rect(-1, -1, 639, 479);
	impl->fullscreen = false;
	impl->flipping_buffers = 2;
	impl->allow_resize = false;
	impl->refresh_rate = 0;
	impl->bpp = 0;
#ifdef WIN32
	impl->layered = false;
#endif
}

CL_DisplayWindowDescription::~CL_DisplayWindowDescription()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// CL_DisplayWindowDescription attributes:

const std::string &CL_DisplayWindowDescription::get_title() const
{
	return impl->title;
}

CL_Size CL_DisplayWindowDescription::get_size() const
{
	return impl->position.get_size();
}

CL_Rect CL_DisplayWindowDescription::get_position() const
{
	return impl->position;
}

bool CL_DisplayWindowDescription::is_fullscreen() const
{
	return impl->fullscreen;
}

int CL_DisplayWindowDescription::get_flipping_buffers() const
{
	return impl->flipping_buffers;
}

bool CL_DisplayWindowDescription::get_allow_resize() const
{
	return impl->allow_resize;
}

#ifdef WIN32
bool CL_DisplayWindowDescription::is_layered() const
{
	return impl->layered;
}
#endif

int CL_DisplayWindowDescription::get_bpp() const
{
	return impl->bpp;
}

int CL_DisplayWindowDescription::get_refresh_rate() const
{
	return impl->refresh_rate;
}

/////////////////////////////////////////////////////////////////////////////
// CL_DisplayWindowDescription operations:

void CL_DisplayWindowDescription::set_title(const std::string &title)
{
	impl->title = title;
}

void CL_DisplayWindowDescription::set_size(const CL_Size &size)
{
	impl->position.set_size(size);
}

void CL_DisplayWindowDescription::set_position(const CL_Rect &position)
{
	impl->position = position;
}

void CL_DisplayWindowDescription::set_fullscreen(bool fullscreen)
{
	impl->fullscreen = fullscreen;
}

void CL_DisplayWindowDescription::set_flipping_buffers(int num_buffers)
{
	impl->flipping_buffers = num_buffers;
}

void CL_DisplayWindowDescription::set_allow_resize(bool allow_resize)
{
	impl->allow_resize = allow_resize;
}

void CL_DisplayWindowDescription::set_bpp(int bytes)
{
	impl->bpp = bytes;
}

void CL_DisplayWindowDescription::set_refresh_rate(int refresh_rate)
{
	impl->refresh_rate = refresh_rate;
}

#ifdef WIN32
void CL_DisplayWindowDescription::set_layered(bool layered)
{
	impl->layered = layered;
}
#endif

/////////////////////////////////////////////////////////////////////////////
// CL_DisplayWindowDescription implementation:
