/*  $Id: graphic_context.cpp,v 1.20 2003/09/12 11:57:29 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Display/display_precomp.h"
#include "API/Display/graphic_context.h"
#include "graphic_context_generic.h"

/////////////////////////////////////////////////////////////////////////////
// CL_GraphicContext construction:

CL_GraphicContext::CL_GraphicContext()
: impl(0)
{
}

CL_GraphicContext::CL_GraphicContext(const CL_GraphicContext &copy)
: impl(copy.impl)
{
	if (impl) impl->add_ref();
}

CL_GraphicContext::CL_GraphicContext(CL_GraphicContext_Generic *impl)
: impl(impl)
{
	if (impl) impl->add_ref();
}

CL_GraphicContext::~CL_GraphicContext()
{
	if (impl) impl->release_ref();
}

/////////////////////////////////////////////////////////////////////////////
// CL_GraphicContext attributes:

int CL_GraphicContext::get_width() const
{
	return impl->get_width();
}

int CL_GraphicContext::get_height() const
{
	return impl->get_height();
}

const CL_Rect &CL_GraphicContext::get_cliprect() const
{
	return impl->get_cliprect();
}

int CL_GraphicContext::get_translate_x() const
{
	return impl->get_translate_x();
}

int CL_GraphicContext::get_translate_y() const
{
	return impl->get_translate_y();
}

CL_Color CL_GraphicContext::get_pixel(int x, int y) const
{
	return impl->get_pixel(x, y);
}

/////////////////////////////////////////////////////////////////////////////
// CL_GraphicContext operations:

CL_GraphicContext &CL_GraphicContext::operator =(const CL_GraphicContext &copy)
{
	if (impl) impl->release_ref();
	impl = copy.impl;
	if (impl) impl->add_ref();
	return *this;
}

void CL_GraphicContext::begin_3d()
{
	impl->begin_3d();
}

void CL_GraphicContext::end_3d()
{
	impl->end_3d();
}

void CL_GraphicContext::flush()
{
	impl->flush();
}

void CL_GraphicContext::set_cliprect(const CL_Rect &rect)
{
	impl->set_cliprect(rect);
}

void CL_GraphicContext::push_cliprect(const CL_Rect &rect)
{
	impl->push_cliprect(rect);
}

void CL_GraphicContext::push_cliprect()
{
	impl->push_cliprect();
}

void CL_GraphicContext::pop_cliprect()
{
	impl->pop_cliprect();
}

void CL_GraphicContext::draw_pixel(int x, int y, const CL_Color &color)
{
	impl->draw_pixel(x, y, color);
}

void CL_GraphicContext::draw_line(int x1, int y1, int x2, int y2, const CL_Color &color)
{
	impl->draw_line(x1, y1, x2, y2, color);
}

void CL_GraphicContext::draw_rect(const CL_Rect &rect, const CL_Color &color)
{
	impl->draw_rect(rect, color);
}

void CL_GraphicContext::fill_rect(const CL_Rect &rect, const CL_Gradient &gradient)
{
	impl->fill_rect(rect, gradient);
}

void CL_GraphicContext::fill_rect(const CL_Rect &rect, const CL_Color &color)
{
	impl->fill_rect(rect, color);
}

void CL_GraphicContext::clear(const CL_Color &color)
{
	impl->clear(color);
}

void CL_GraphicContext::set_translate_offset(int x, int y)
{
	impl->set_translate_offset(x, y);
}

void CL_GraphicContext::push_translate_offset(int delta_x, int delta_y)
{
	impl->push_translate_offset(delta_x, delta_y);
}

void CL_GraphicContext::push_translate_offset()
{
	impl->push_translate_offset();
}

void CL_GraphicContext::pop_translate_offset()
{
	impl->pop_translate_offset();
}

/////////////////////////////////////////////////////////////////////////////
// Implementation:
