/*  $Id: resourcedata_surface.cpp,v 1.10 2003/09/03 22:58:17 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "Display/display_precomp.h"
#include "resourcedata_surface.h"
#include "API/Core/Resources/resource_manager.h"
#include "API/Display/Providers/provider_factory.h"

//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Surface construction:

CL_ResourceData_Surface::CL_ResourceData_Surface(CL_Resource &resource)
: CL_ResourceData(resource)
{
	resource.attach_data("surface", this);
}

CL_ResourceData_Surface::~CL_ResourceData_Surface()
{
}
	
//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Surface attributes:


//////////////////////////////////////////////////////////////////////////////
// CL_ResourceData_Surface implementation:

void CL_ResourceData_Surface::on_load()
{
	CL_Resource resource = get_resource();
	std::string name = resource.get_element().get_attribute("file");
	std::string image_format = resource.get_element().get_attribute("format");
	std::string str_hint = resource.get_element().get_attribute("hint", "automatic");

	CL_Surface::Hint hint = CL_Surface::hint_automatic;
	if (str_hint == "automatic") hint = CL_Surface::hint_automatic;
	else if (str_hint == "opaque") hint = CL_Surface::hint_opaque;
	else if (str_hint == "colorkey") hint = CL_Surface::hint_colorkey;
	else if (str_hint == "alpha") hint = CL_Surface::hint_alpha;

	CL_PixelBuffer *image = CL_ProviderFactory::load(name, image_format, resource.get_manager().get_resource_provider());
	surface = CL_Surface(image, true, hint);
}
	
void CL_ResourceData_Surface::on_unload()
{
	surface = CL_Surface();
}
