/*  $Id: menudata.cpp,v 1.14 2003/07/20 22:23:22 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"

#include "API/GUI/label.h"
#include "API/GUI/menudata.h"
#include "API/GUI/menunode.h"
#include "API/GUI/popupmenu.h"
#include "menudata_generic.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_MenuData::CL_MenuData(CL_PopupMenu *popupmenu)
{
	impl = new CL_MenuData_Generic(this, popupmenu);
}

CL_MenuData::~CL_MenuData()
{
	clear();
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

const std::list<CL_MenuNode *> &CL_MenuData::get_items() const
{
	return impl->items;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

CL_MenuNode *CL_MenuData::insert_item(const std::string &text, int index)
{
	return insert_item(text, NULL, index);
}

CL_MenuNode *CL_MenuData::insert_item(const std::string &text, CL_PopupMenu *submenu, int index)
{
	CL_Component *new_component = new CL_Label(text, impl->popupmenu->get_client_area());

	return insert_item(new_component, submenu, true, index);
}

CL_MenuNode *CL_MenuData::insert_item(CL_Component *new_component, bool delete_component, int index)
{
	return insert_item(new_component, NULL, delete_component, index);
}

CL_MenuNode *CL_MenuData::insert_item(CL_Component *new_component, CL_PopupMenu *submenu, bool delete_component, int index)
{
	// Create menu node
	CL_MenuNode *node = new CL_MenuNode(submenu);
	impl->items.push_back(node);

	// Add component to hierarchy
	impl->popupmenu->get_client_area()->add_child(new_component);
	node->set_component(new_component, delete_component);

	// Make sure components get positioned correctly
	impl->popupmenu->sig_resize()(0,0);

	return node;
}

void CL_MenuData::clear()
{
	std::list<CL_MenuNode *>::iterator it;
	for (it = impl->items.begin(); it != impl->items.end(); ++it)
		delete (*it);

	impl->items.clear();
}
