/*  $Id: menunode.cpp,v 1.5 2003/07/20 22:23:22 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"

#include "API/GUI/component.h"
#include "API/GUI/menunode.h"
#include "menunode_generic.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_MenuNode::CL_MenuNode(CL_PopupMenu *submenu)
{
	impl = new CL_MenuNode_Generic(this, submenu);
}

CL_MenuNode::~CL_MenuNode()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

bool CL_MenuNode::has_submenu() const
{
	return (impl->submenu != NULL);
}

void *CL_MenuNode::get_userdata() const
{
	return impl->userdata;
}

CL_Component *CL_MenuNode::get_component() const
{
	return impl->component;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_MenuNode::set_component(CL_Component *component, bool delete_component)
{
	if(impl->component)
	{
		// TODO: Disconnect the signal:
		// impl->slots.connect(new_component->sig_mouse_up(), impl, &CL_MenuNode_Generic::on_child_click);

		if(impl->delete_component)
			delete impl->component;
	}

	impl->component = component;
	impl->delete_component = delete_component;

	impl->slots.connect(component->sig_mouse_up(), impl, &CL_MenuNode_Generic::on_child_click);
}

void CL_MenuNode::set_userdata(void *data)
{
	impl->userdata = data;
}
