/*  $Id: popupmenu_generic.cpp,v 1.27 2003/09/18 17:53:29 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "popupmenu_generic.h"
#include "API/GUI/gui_manager.h"
#include "API/Core/XML/dom_element.h"
#include "API/Core/System/log.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_PopupMenu_Generic::CL_PopupMenu_Generic(CL_PopupMenu *self)
: popupmenu(self)
{
	// Create client area (which is the main popupmenu)
	CL_Component *client_area = new CL_Component(popupmenu);
	popupmenu->set_client_area(client_area);
	
	slot_resize = popupmenu->sig_resize().connect(
		this, &CL_PopupMenu_Generic::on_resize);

	on_resize(popupmenu->get_width(), popupmenu->get_height());
}

CL_PopupMenu_Generic::~CL_PopupMenu_Generic()
{
	delete popupmenu->get_client_area();
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_PopupMenu_Generic::popup(CL_Component *parent)
{
	slot_mouse_down = parent->sig_mouse_down().connect(
		this, &CL_PopupMenu_Generic::on_mouse_down);
	slot_lost_focus = popupmenu->get_client_area()->sig_lost_focus().connect(
		this, &CL_PopupMenu_Generic::on_lost_focus);

	// Alt.1: Run popupmenu as a modal component
//	popupmenu->run(parent);

	// Alt.2: Run popupmenu as a captured component
//	parent->add_child(popupmenu);
//	popupmenu->capture_mouse();
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_PopupMenu_Generic::on_mouse_down(const CL_InputEvent &key)
{
//	CL_Log::log("debug", "CL_PopupMenu_Generic::on_mouse_down");
	
/*	CL_Rect pos = popupmenu->get_screen_rect();
	if(key.x < pos.x1 || key.y < pos.y1 || key.x > pos.x2 || key.y > pos.y2)
	{
		sig_cancelled();
		popupmenu->close();
	}	
*/
}

void CL_PopupMenu_Generic::on_resize(int old_width, int old_height)
{
	int width = popupmenu->get_width();
	int height = popupmenu->get_height();

	CL_Rect rect(4, 4, 4 + width - 8, 4 + height - 8);
	popupmenu->get_client_area()->set_position(rect);
}

void CL_PopupMenu_Generic::on_lost_focus()
{
//	CL_Log::log("debug", "CL_PopupMenu_Generic::on_lost_focus");
}
