/*  $Id: treeitem.cpp,v 1.4 2003/07/20 22:23:22 grumbel Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"

#include "treeitem_generic.h"
#include "component_generic.h"
#include "API/GUI/stylemanager.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_TreeItem::CL_TreeItem(
	CL_TreeNode *node,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(parent, style), impl(NULL)
{
	impl = new CL_TreeItem_Generic(this, node, "");
	get_style_manager()->connect_styles("treeview_item", this);
	find_preferred_size();
}

CL_TreeItem::CL_TreeItem(
	CL_TreeNode *node,
	const std::string &text,
	CL_Component *parent,
	CL_StyleManager *style)
: CL_Component(parent, style), impl(NULL)
{
	impl = new CL_TreeItem_Generic(this, node, text);
	get_style_manager()->connect_styles("treeview_item", this);
	find_preferred_size();
}

CL_TreeItem::~CL_TreeItem()
{
	delete impl;
}

/////////////////////////////////////////////////////////////////////////////
// Attributes:

const std::string &CL_TreeItem::get_text() const
{
	return impl->text;
}

CL_TreeNode *CL_TreeItem::get_node() const
{
	return impl->node;
}

/////////////////////////////////////////////////////////////////////////////
// Operations:

void CL_TreeItem::set_text(const std::string &text)
{
	impl->text = text;
}

/////////////////////////////////////////////////////////////////////////////
// Signals:

CL_Signal_v0 &CL_TreeItem::sig_clicked()
{
	return impl->sig_clicked;
}

