/*  $Id: treeview_generic.cpp,v 1.16 2003/09/18 17:53:29 mbn Exp $
**
**  ClanLib Game SDK
**  Copyright (C) 2003  The ClanLib Team
**  For a total list of contributers see the file CREDITS.
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Lesser General Public License for more details.
**
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
**
*/

#include "precomp.h"
#include "treeview_generic.h"
#include "API/GUI/gui_manager.h"
#include "API/Core/XML/dom_element.h"

/////////////////////////////////////////////////////////////////////////////
// Construction:

CL_TreeView_Generic::CL_TreeView_Generic(CL_TreeView *self)
: treeview(self)
{
	// Create client area (which is the main treeview)
	CL_Component *client_area = new CL_Component(treeview);
	treeview->set_client_area(client_area);
	
	slot_paint_children = client_area->sig_paint_children().connect_virtual(
		this, &CL_TreeView_Generic::on_paint_children);
	slot_resize = treeview->sig_resize().connect(
		this, &CL_TreeView_Generic::on_resize);

	on_resize(treeview->get_width(), treeview->get_height());
}

CL_TreeView_Generic::~CL_TreeView_Generic()
{
	delete treeview->get_client_area();
}

/////////////////////////////////////////////////////////////////////////////
// Callbacks:

void CL_TreeView_Generic::on_paint_children(CL_SlotParent_v0 &super)
{
	CL_Point point;
	treeview->draw_nodes(point);
}

void CL_TreeView_Generic::on_resize(int old_width, int old_height)
{
	int width = treeview->get_width();
	int height = treeview->get_height();

	CL_Rect rect(4, 4, 4 + width - 8, 4 + height - 8);
	treeview->get_client_area()->set_position(rect);
}
